-----------------------------------------------------------------------------------------------
-- Client Lua Script for ShowOff
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "Apollo"
require "ChatSystemLib"
require "AchievementsLib"

-----------------------------------------------------------------------------------------------
-- ShowOff Module Definition
-----------------------------------------------------------------------------------------------
local ShowOff = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
-- constant for the addon name to include in notifications
local soShowOffNotificationTag = "<ShowOff>"
-- constant for the spacing to use in padding notifications
local soShowOffNotificationSpacerText = "  "
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ShowOff:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function ShowOff:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ShowOff OnLoad
-----------------------------------------------------------------------------------------------
function ShowOff:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ShowOff.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ShowOff OnDocLoaded
-----------------------------------------------------------------------------------------------
function ShowOff:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ShowOffForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("showoff", "OnShowOffOn", self)
		
		-- Call OnLevelUp when player levels up
		Apollo.RegisterEventHandler("PlayerLevelChange","OnLevelUp", self)
		Apollo.RegisterEventHandler("AchievementGranted","OnAchievementGranted", self)
		Apollo.RegisterEventHandler("PlayedTime","OnPlayedTime", self)
-----TODO:  REMOVE THE FOLLOWING WHICH WAS USED FOR TESTING
--		Apollo.RegisterEventHandler("Mount","OnAchievementGranted", self)


		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- ShowOff OnLevelUp
-----------------------------------------------------------------------------------------------
-- handles the OnLevelUp event and sends the configured notifications
function ShowOff:OnLevelUp()	
	--notify the user's notification groups of a level up
	---determine if the user is in a pvp match
	local strNotificationMessage = ""
	if MatchingGame.IsInMatchingGame() or MatchingGame.IsInPVPGame() then
		strNotificationMessage =  soShowOffNotificationTag..soShowOffNotificationSpacerText.."DING!  Oh shit, cupcake! I levelled up in PVP!"
	else
		strNotificationMessage =  soShowOffNotificationTag..soShowOffNotificationSpacerText.."DING!  I am now level "..GameLib.GetPlayerLevel().."!"
	end
	
	--broadcast the message
	self:BroadcastMessage(strNotificationMessage)
end

-----------------------------------------------------------------------------------------------
-- ShowOff OnAchievementGranted
-----------------------------------------------------------------------------------------------
--handles the OnAchievementGranted event and sends the configured notifications
function ShowOff:OnAchievementGranted(achComplete)
	--broadcast the message publically if the achievement is worth more than zero (0) pts
	if achComplete:GetPoints() > 0 then
		--format the notification message
		local strBroadcastMessage = soShowOffNotificationTag..soShowOffNotificationSpacerText.."Achievement Granted! ("..achComplete:GetName().." - "..achComplete:GetPoints().."pts) Total pts: "..AchievementsLib.GetAchievementPoints()+achComplete:GetPoints() 	
		--broadcast the message
		self:BroadcastMessage(strBroadcastMessage)
	else
		--format the notification message
		local strBroadcastMessage = soShowOffNotificationTag..soShowOffNotificationSpacerText.."Achievement Granted! ("..achComplete:GetName().." - "..achComplete:GetPoints().."pts) Total pts: "..AchievementsLib.GetAchievementPoints()+achComplete:GetPoints().."  This achievement has been suppressed since it is not worth any achievement points and is not worth showing off. Sorry, cupcake."

		--broadcast the message	
		self:BroadcastMessageLocal(strBroadcastMessage)
	end
	
end

--handles the PlayedTime event and sends the configured notifications
function ShowOff:OnPlayedTime(soPlayedTime, soTotalTimePlayed, soTimePlayedOnThisLevel, soTimePlayedThisSession)
--	for k,v in pairs(soPlayedTime) do
---		self:NotifySayChannel(v)
   -- end
		--soTotalTimePlayed is the number of days, hours, minutes, seconds that the character has been played
		--   eg) Total time played: 1 days, 5 hours, 17 minutes, 2 seconds.
		local playedTimeConstant = "Total time played: "
		local playedTimeDaysToken = "day"
		local playedTimeSubStr = string.sub(soTotalTimePlayed, 1, string.find(soTotalTimePlayed,playedTimeDaysToken)-1)
		playedTimeSubStr = string.sub(playedTimeSubStr, string.len(playedTimeConstant))
		playedTimeSubStr = string.gsub(playedTimeSubStr, "%s", "")

		
--		self:NotifySayChannel("__"..playedTimeSubStr.."__")


--		self:NotifySayChannel(string.match(soTotalTimePlayed,)

end 

-----------------------------------------------------------------------------------------------
-- ShowOff Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/showoff"
function ShowOff:OnShowOffOn()
	self.wndMain:Invoke() -- show the window
end

--broadcasts the specified message to the system channel
function ShowOff:NotifySystemChannel(strBroadcastMessage)
	local strMessageToDisplay = ""
	if strBroadcastMessage~= nil then
		strMessageToDisplay = strBroadcastMessage
	end
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, strMessageToDisplay)
end

--broadcasts the specified message to the say channel
function ShowOff:NotifySayChannel(strBroadcastMessage)
	local strMessageToDisplay = ""
	if strBroadcastMessage~= nil then
		strMessageToDisplay = strBroadcastMessage
	end
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Say, strMessageToDisplay)
end

--broadcasts the specified message to the guild channel
function ShowOff:NotifyGuildChannel(strMessage)
	---TODO: Gotta be a better way! There doesn't seem to be a better way of getting the Guild chat channel
	for _,channel in pairs(ChatSystemLib.GetChannels()) do
		---TODO: Verify that the user is in a guild
    	if channel:GetType() == ChatSystemLib.ChatChannel_Guild then
        	channel:Send(strMessage)
			return				--exits the for loop
		end
    end
end

--broadcast the message to the configured channels
function ShowOff:BroadcastMessage(strBroadcastMessage)
	--broadcast to say chat
	self:NotifySayChannel(strBroadcastMessage)
			
	--broadcast to guild chat
	self:NotifyGuildChannel(strBroadcastMessage)
end

--broadcast the message locally
function ShowOff:BroadcastMessageLocal(strBroadcastMessage)
	self:NotifySystemChannel(strBroadcastMessage)
end

-----------------------------------------------------------------------------------------------
-- ShowOffForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function ShowOff:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function ShowOff:OnCancel()
	self.wndMain:Close() -- hide the window
end


-----------------------------------------------------------------------------------------------
-- ShowOff Instance
-----------------------------------------------------------------------------------------------
local ShowOffInst = ShowOff:new()
ShowOffInst:Init()
